/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is APNG Edit.
 *
 * The Initial Developer of the Original Code is Mozilla.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Justin Dolske <dolske@mozilla.com> (Original Author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function Startup() {
    APNGedit.init();
    APNGui.init();
}

function Shutdown() {
}

var APNGui = {
    _warnOnQuit : false,

    init : function () {
        this.updateAnimationInfobox();
    },


    Quit : function () {
        var ok = true;
        if (this._warnOnQuit) {
            // XXX use prompt service to avoid "[Javascript Application]"
            // (why is that even being displayed, since we're chrome?!)
            ok = window.confirm("Quit without saving changes?");
        }
        if (ok) { window.close(); }
    },


    updateAnimationInfobox: function () {
        var e;
        e = document.getElementById("info_frames");
        var numFrames = APNGedit._lightbox.childNodes.length - 1;
        e.value = numFrames + " frames";

        // Only show "save" buttons when there's something to do.
        document.getElementById("save_ani").disabled = (numFrames == 0);
        document.getElementById("save_fra").disabled = (numFrames == 0);

        e = document.getElementById("info_length");
        var length = 0;
        for (var i = 0; i < APNGedit._lightbox.childNodes.length - 1; i++) {
            if (APNGedit.aniInfo.skipFirstFrame && i == 0) { continue; }
            length += APNGedit._lightbox.childNodes[i].delay;
        }
        e.value = length + " msec";

        // The animation SRC attribute is a data: URL, get size by adjusting for Base64 encoding
        // overhead. (Ignoring padding, so we might be off by a few bytes)
        e = document.getElementById("info_size");
        var ani = document.getElementById("apng");
        var size = ani.src.length - "data:image/png;base64,".length;
        var size = Math.round((size / 4) * 3 / 100) / 10; // Get one decimal place.
        e.value = size + " KB";
    },


    SaveAnimation : function() {
        //var filename = prompt("Enter filename to save as:");
        //if (!filename) { return; }
        var url = document.getElementById("apng").src;
        var filename   = "animation.png";
        var skipCache  = true;
        var skipPrompt = false;
        var referrer   = null;
        saveImageURL(url, filename, "SaveImageTitle", skipCache, skipPrompt, referrer);
        //APNGedit.saveAnimation(filename);
    },


    SaveFrames : function() {
        APNGedit.log("Saving all frames");
        var numFrames = APNGedit._lightbox.childNodes.length - 1;
        var ok = window.confirm("Are you sure you want to save all " + numFrames + " frames?");
        if (!ok)
            return;

        var tmpCanvas = APNGedit._tmpCanvas;
        var tmpCtx = tmpCanvas.getContext("2d");

        for (var i = 0; i < numFrames; i++) {
            APNGedit.log("Saving frame " + (i+1));
            var frame = APNGedit._lightbox.childNodes[i];
            var w = frame.pixelsW;
            var h = frame.pixelsH;
            tmpCanvas.width  = w;
            tmpCanvas.height = h;
            tmpCtx.putImageData({width : w, height : h, data : frame.pixels}, 0, 0);

            var url = tmpCanvas.toDataURL();
            var filename = "frame" + (i < 9 ? "0" : "") + (i+1) + ".png";

            var skipCache  = true;
            var skipPrompt = true;
            var referrer   = null;
            saveImageURL(url, filename, "SaveImageTitle", skipCache, skipPrompt, referrer);
        }

    },

    RunScript : function () {

        if (1) {
            var scriptURL = this.PickFile("scripts");
            if (!scriptURL)
                return;

            // We might be slow, so start a timer
            APNGedit._slowID = window.setTimeout(APNGedit.showSlowUI, 1500);

            APNGui.oldAutoPreview = APNGedit.autoPreview;
            APNGedit.setAutoPreview(false);
            try {
                var svc = Cc["@mozilla.org/moz/jssubscript-loader;1"].
                          getService(Ci.mozIJSSubScriptLoader);
                svc.loadSubScript(scriptURL);
            } catch (e) {
                APNGedit.log("subscript loader failed: " + e);
            }

            setTimeout(APNGedit.do_render_frame, 5, 0);
            return;
        }


        function safe_log(aString) {
            if (typeof aString != "string")
                aString = new XPCSafeJSObjectWrapper(aString).toString();

            APNGedit.log("SCRIPT: " + aString);
        }

        function safe_getFrameCanvas(aIndex) {
            if (typeof aIndex != "number")
                throw "getFrameCanvas didn't get a number";

            APNGedit.log("getFrameCanvas: " + aIndex);

            var canvas = APNGedit._lightbox.childNodes[0];

            // XXX bah. this doesn't work, "[Exception... 'Method not implemented'" thrown.
            // Tried unwrapped childNodes[0], childNodes[0]._canvas, childNodes[0].canvas
            // Altough childNodes[0].framenum works.

            return new XPCNativeWrapper(canvas);
        }

        var scriptURL = this.PickFile("scripts", true);
        if (!scriptURL)
            return;

        APNGedit.log("Running script: " + scriptURL);

        // Get the string contents of the script file.
        var file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
        file.initWithPath(scriptURL);

        inputStream = Cc["@mozilla.org/network/file-input-stream;1"].
                      createInstance(Ci.nsIFileInputStream);
        inputStream.init(file, 0x01, 0600, null); // RD_ONLY
        lineStream = inputStream.QueryInterface(Ci.nsILineInputStream);

        var scriptText = "";
        var line = { value : "" };
        while (hasMore = lineStream.readLine(line)) { scriptText += line.value; }


        // create a sandbox with a given principal
        var s = Components.utils.Sandbox("http://www.example.com/");
        s.importFunction(safe_log, "log");
        s.importFunction(safe_getFrameCanvas, "getFrameCanvas");
        var frames = new XPCNativeWrapper(APNGedit._lightbox.childNodes);
        s.frames = frames;

        Components.utils.evalInSandbox(scriptText, s);
        APNGedit.log("script done");

    },


    PickFile : function (type, asPathname) {
        var url = null;
        var path = null;

        // from http://mxr.mozilla.org/seamonkey/source/browser/base/content/browser.js BrowserOpenFileWindow()
        try {
            const nsIFilePicker = Components.interfaces.nsIFilePicker;
            var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
            fp.init(window, "Open File", nsIFilePicker.modeOpen);
            if (type == "images")
                fp.appendFilters(nsIFilePicker.filterImages);
            else if (type == "scripts")
                fp.appendFilter("Javascript", "*.js");

            if (fp.show() == nsIFilePicker.returnOK) {
                url = fp.fileURL.spec;
                path = fp.fileURL.path;
            }
        } catch (ex) { }

        //url = window.prompt("Enter a URL:");

        return (asPathname ? path : url);
    },


    setPreviewBackground : function (style) {
        const checkersURL = "url(chrome://apngedit/skin/checker.png)";
        var color = "";
        var image = "";

        APNGedit.log("Changing BG preview style to " + style);

        switch (style) {
            case "black":     color = "#000000"; break;
            case "darkGrey":  color = "#202020"; break;
            case "liteGrey":  color = "#808080"; break;
            case "white":     color = "#ffffff"; break;
            case "red":       color = "#ff0000"; break;
            case "green":     color = "#00ff00"; break;
            case "blue":      color = "#0000ff"; break;

            case "checkers":
            default:
                image = checkersURL;
                break;
        }

        APNGedit.ani.style.backgroundColor = color; 
        APNGedit.ani.style.backgroundImage = image; 

    },


    loopingUpdate : function () {
        var stale = false;

        // Check for loop count update
        var e = document.getElementById("loopRadioGroup");
        var newPlayCount = e.selectedItem.value;
        if (newPlayCount == 2) {
            e = document.getElementById("playCountTextbox");
            newPlayCount = e.value;
        }
        if (newPlayCount != APNGedit.aniInfo.playCount) {
            stale = true;
            APNGedit.aniInfo.playCount = newPlayCount;
        }
    
        // Check for skip first frame update
        var e = document.getElementById("skip");
        var newSkip = e.checked;
        if (newSkip != APNGedit.aniInfo.skipFirstFrame) {
            stale = true;
            APNGedit.aniInfo.skipFirstFrame = newSkip;
        }

        // Update animation if needed
        if (stale) {
            APNGedit.updateAnimation();
        }
    }
};
